(define *assertion* #f)
(define *main-context-name* #f)
(define *main-assertion-name* #f)

(front-end/add-simple-option! "Assertion" "--assertion=<name>"
															"Qualified assertion name using the LSAL syntax. Unfortunately, the SAL official syntax does not define qualified assertion names. Qualified assertion names are useful to reference properties in parametric contexts. Examples of qualified assertion names:\n(@ at-most-one-ack (arbiter () (50)))\n(@ invalid mutex)"
															(lambda (arg) (set! *assertion* arg)))

(define (print-error . args)
	(with-output-to-port (current-error-port)
		(lambda ()
			(apply print "Error: " args))))

(define (print-path-core path)
	(sal-path/display path))

(define (print-path path)
	(print-path-core path))

(define (print-counter-example counter-example)
	(print "Counterexample:")
	(print-path-core counter-example))

(define (check-if-supported-assertion assertion)
	(unless (instance-of? assertion <sal-module-models>)
		(print-error "Only simple assertions (MODULE-MODELS) are supported in the current version.")
		(exit -1)))

(define (check-assertion-name-ref help-proc)
	(when (and *assertion* (or *main-context-name* *main-assertion-name*))
		(print-error "Invalid combination of arguments, you must not specify context and assertion names when the option --assertion is used.")
		(help-proc))
	(unless (or *assertion* (and *main-context-name* *main-assertion-name*))
		(print-error "An assertion must be specified.")
		(help-proc)))

(define (check-if-trivial-assertion assertion)
	(when (sal-module-models/trivially-true? assertion)
		(warning-message "The property is trivially true.")
		(print "proved.")
		(exit 0))
	(when (sal-module-models/trivially-false? assertion)
		(warning-message "The property is trivially false.")
		(print "invalid.")
		(exit 0)))

(define (guess-qualified-name-parser! qualified-name)
	(if (substring=? qualified-name "(@" 2)
		(sal/set-make-sal-string-reader-proc! make-lsal-string-reader)
		(sal/set-make-sal-string-reader-proc! make-sal-string-reader)))

(define (load-context-if-file-name context)
	(let ((ctx-name (file-name->sal-context-name context)))
		(unless (equal? context ctx-name)
			(sal-env/import-context-from-file! *sal-env* context))))